/*
This file defines three classes related to hardware configurations : 
	CChannelConfig - settings for 1 channel within a hardware configuration
	CConfiguration - Collection of AI and AO settings 
	CConfigurations - array of CConfiguration 
	
File Format V. 2 (1st real version?)
[QCNF][int iVer=2][int iPos][int iConfigs][int iSetPos]
V.3 adds : 
[QCNF][int iVer=3][int iPos][int iConfigs][int iSetPos]
	[int iGainTablePos][int iGainTableCount]
	
	
*/

const int CONFIG_AIROWS = 32;			// fixed number of analog input control rows
const int CONFIG_AOROWS = 4;			// fixed number of analog output control rows

const int CONFIG_DIROWS = 16;
const int CONFIG_DOROWS = 16;

const int GAIN_MAXENTRIES = 10;			// Entries per gain table - limited by number of controls on dialogue.

extern char * g_apchDigMode[5];
extern char * g_ConfigLoc;

class CGainTable : public CItem { 
	public : 
		CString m_cTableName;
		int m_iEntries;
		double m_adSignal[GAIN_MAXENTRIES];
		double m_adMultiplier[GAIN_MAXENTRIES];
		
		double m_adMin[GAIN_MAXENTRIES];	// min/max Filled in by SetRanges() - range this entry applies to.
		double m_adMax[GAIN_MAXENTRIES];	
		int m_iCurrent;						// Last Entry returned by Multiplier - best candidate for next lookup.

		bool m_bNew;						// For window opening, true indicates this is a new entry and [Delete] is not available.
		
		CGainTable();						// Construct
		void SetRanges();					// Fill in m_adMin, m_adMax;
		double Multiplier( double dSignal );	//-- look up multiplier and range of values that it applies to.		
			
		bool LoadFromFile( FILE * h, int iVer );
		bool SaveToFile( FILE * h );
		int SaveSize();
		
	};
	
class CChannelConfig { 
	public : 
		int m_iActive;
		CString m_cChannelName;
		double m_dGain;
		CString m_cUnits;
		double m_dScaling;

		bool LoadFromFile( FILE * h );
		bool SaveToFile( FILE * h );
		int SaveSize();
		
		void Clear();
		CChannelConfig();
	};

class CConfiguration : public CItem { 
	public : 
		CString m_cConfigName;
		CString m_cDeviceName;
		CChannelConfig m_aoInChnCfg[CONFIG_AIROWS];
		CChannelConfig m_aoOutChnCfg[CONFIG_AOROWS];
		CString m_acDigName[CONFIG_DOROWS];
		BYTE m_aiDigMode[CONFIG_DOROWS];
		int m_iAcqMeasMode;

		bool LoadFromFile( FILE * h, int iVer );
		bool SaveToFile( FILE * h );
		int SaveSize();
		
		CConfiguration() ;
		int ActiveAcqChannels( );
		int ActiveGenChannels( );
		bool HasProtocolDigGen();				// true if any m_aiDigMode is set to 'Protocol'
		int DigGenProtocolCount( BYTE ** paiDigGenMap );	// count of m_aiDigMode = Protocol, creat emap of used bits
		int DigAcqCount( BYTE ** paiDigAcqMap = NULL );	// count of m_aiDigMode = Input, create map of used bits
	};

class CConfigurations { 
	public : 
		//-- Array of Configurations 
		CItemArray m_aConfigs;
		CConfiguration * Config( int i );
		int ConfigsSize();

		CItemArray m_aGainTables;
		CGainTable * GainTable( int i );
		CGainTable * GainTable( const char * pchTableName );
		int GainTableSize();

		CItemArray m_aFeedbackConfigs;
		CFeedback * FeedbackConfig( int i );
		CFeedback * FeedbackConfig( const char * pchFeedbackConfigName );
		int FeedbackConfigSize();

		CQUBIOSettings m_oSettings;

		//-- persistence		
		bool LoadFile( );
		bool SaveFile( ) ;
		bool ConfigFileExists( ) ;
		
		//-- manipulate list of configs
		bool AddConfig( CConfiguration * poConfig );
		bool DeleteConfig( const char * pchConfigName );
		bool DeleteConfig( int iConfig );
		int FindConfig( const char * pchConfigName );
	};
		

